
if($ARGV[0] eq ""){
	print("Usage:\nperl raw_spi.pl <serial port>\n\n");
	exit(-1);
	}

$cardname = "CANSNF";
$serial_port = $ARGV[0];



######################################################################
# init serial stuff
print("Init serial port \n");
@rc = SETUP_SERIAL($serial_port);
if($rc[0] != 0){
	exit($rc[0]);
}
$COMPORT = $rc[1];

#flush serial port buffers
$sendstring = "         ";	# garbage chars at start are to just flush out port
$count_out = $COMPORT->write($sendstring);
if(length($sendstring) != $count_out){
	print("Error send failed!!\n",$count_out," Sent out of ",length($sendstring),"\n");
	exit(-1);
}
# this code clears the read buffer (just read all the chars in until its empty)
$recv_length = 1;
$count_in = 1;
while ($count_in == 1){						
	($count_in, $recvstring) = $COMPORT->read($recv_length);
}

######################################################################
# Reset card
print("	Done\nReset card \n");
$sendstring = $cardname.chr(0x00); 	# send reset command to box
$count_out = $COMPORT->write($sendstring);
if(length($sendstring) != $count_out){
	print("TX error could not init box for some reason...\n");
	exit(-1);
}
$recv_length = 7;						# how many chars to get
($count_in, $recvstring) = $COMPORT->read($recv_length);
if($count_in != $recv_length || $recvstring ne ("RESET".chr(0x0D).chr(0x0A))){	
	print("RX cnt error could not init box for some reason...\n");
	print("count: ",$count_in,"\n");
	print("data: \n",unpack("H*",$recvstring),"\n");
	exit(-1);
}

######################################################################
# read firmare level
print("	Done\nRead inital firmware level\n");
$sendstring = $cardname.chr(0x01); 	# send version command
$count_out = $COMPORT->write($sendstring);
if(length($sendstring) != $count_out){
	print("version level TX error ..\n");
	exit(-1);
}
$recv_length = 8;
($count_in, $recvstring) = $COMPORT->read($recv_length);
if($count_in != $recv_length || substr($recvstring,0,1) ne "V" || substr($recvstring,6,2) ne chr(0x0D).chr(0x0A)){	
	print("version level RX error...\n");
	print("count: ",$count_in,"\n");
	print("data: \n",unpack("H*",$recvstring),"\n");
	exit(-1);
}
print("	Level = ",substr($recvstring,0,6),"\n");



######################################################################
# raw SPI command
print("	Done\nRaw SPI reset\n");
$rc = &RAW_SPI($COMPORT, "C0");
print("	",$rc,"\n");

print("	Done\nRaw SPI write\n");
$rc = &RAW_SPI($COMPORT, "020001234567");
print("	",$rc,"\n");

print("	Done\nRaw SPI read\n");
$rc = &RAW_SPI($COMPORT, "030000000000");
print("	",$rc,"\n");

print("	Done\nRaw SPI read status\n");
$rc = &RAW_SPI($COMPORT, "A000");
print("	",$rc,"\n");

print("	Done\nSet up CNF1-3\n");
# bit time is 10us for the entertainment bus on my 2008 GTI. Input osc is 20MHz see data sheet for how I got below... (first guess)
$rc = &RAW_SPI($COMPORT, "022807ff43");	# CNF1: SJW = 2TQ, BRP = 3 CNF2: BTLMODE = set by CNF3, SAM = 3 samples, PS1 = 8TQ, PropSeg = 8TQ CNF3: clkout, no wake up filter, PS2 = 8TQ

print("	",$rc,"\n");

print("	Done\nSet up TXRTSCTRL\n");
$rc = &RAW_SPI($COMPORT, "020D00");	# set all pins to inputs
print("	",$rc,"\n");

# set up filters here (if wanted going into promisc mode so will just grab all for now)

print("	Done\nSet up recieve masks\n");
$rc = &RAW_SPI($COMPORT, "022000000000");	# set all pins to inputs
print("	",$rc,"\n");


# all registers above can ONLY be written in config mode...

print("	Done\nSet up RXB0CTRL\n");
$rc = &RAW_SPI($COMPORT, "026066");	# set recieve all messages and roll over to buffer 1
print("	",$rc,"\n");

print("	Done\nSet up BFPCTRL\n");
$rc = &RAW_SPI($COMPORT, "020C00");	# disable buffer full outputs. 
print("	",$rc,"\n");

# switch over to lissen mode..

print("	Done\nread current state\n");
$rc = &RAW_SPI($COMPORT, "030E00");	# see what the current mode is 
print("	",$rc,"\n");

print("	Done\nRequest listen only mode\n");
$rc = &RAW_SPI($COMPORT, "020F60");	# disable buffer full outputs. 
print("	",$rc,"\n");

$rc = &RAW_SPI($COMPORT, "030E00");	# see what the current mode is 
while($rc !~ m/^....6./i){
	$rc = &RAW_SPI($COMPORT, "030E00");	# see what the current mode is 
}
print("	",$rc,"\n");

my $cnt = 0;

while(1){
	$rc = &RAW_SPI($COMPORT, "032c00");	# get IRQ reg
	if($rc !~ m/^....00/i){
		print($cnt." ",substr($rc, 4, 2),"\n");
		$cnt++;
	}
	$rc = &RAW_SPI($COMPORT, "022c00");	# clear all irq

}


print("	Done\n");
exit(0);
###############################################
# this function sends and rcvs the raw SPI data 
###############################################
sub RAW_SPI{
	my $COMPORT = $_[0];
	my $spi_data = $_[1];
	
	my $sendstring = "";
	my $i = "";
	my $count_out = "";
	my $recv_length = "";
	my $count_in = "";
	my $recvstring = "";
	


	######################################################################
	# clear CS
	$sendstring = $cardname.chr(0x06).chr(0x02); 	# send version command
	$count_out = $COMPORT->write($sendstring);
	if(length($sendstring) != $count_out){
		print("clr cs TX error ..\n");
		exit(-1);
	}
	$recv_length = 3;
	($count_in, $recvstring) = $COMPORT->read($recv_length);
	if($count_in != $recv_length || substr($recvstring,1,2) ne chr(0x0D).chr(0x0A)){	
		print("clr cs RX error...\n");
		print("count: ",$count_in,"\n");
		print("data: \n",unpack("H*",$recvstring),"\n");
		exit(-1);
	}

	######################################################################
	# set CS
	$sendstring = $cardname.chr(0x06).chr(0x00); 	# send version command
	$count_out = $COMPORT->write($sendstring);
	if(length($sendstring) != $count_out){
		print("set cs TX error ..\n");
		exit(-1);
	}
	$recv_length = 3;
	($count_in, $recvstring) = $COMPORT->read($recv_length);
	if($count_in != $recv_length || substr($recvstring,1,2) ne chr(0x0D).chr(0x0A)){	
		print("set cs RX error...\n");
		print("count: ",$count_in,"\n");
		print("data: \n",unpack("H*",$recvstring),"\n");
		exit(-1);
	}


	$sendstring = "";
	for($i = 0; $i < length($spi_data); $i = $i + 2){
		$sendstring .= chr(hex(substr($spi_data, $i, 2)));
	}
	$sendstring = $cardname.chr(0x07).chr(length($spi_data)/2-1).$sendstring; 	# send spi command (07) length and data
	$count_out = $COMPORT->write($sendstring);
	if(length($sendstring) != $count_out){
		print("Raw SPI TX error ..\n");
		exit(-1);
	}
	$recv_length = length($spi_data)/2 + 2;
	($count_in, $recvstring) = $COMPORT->read($recv_length);
	if($count_in != $recv_length || substr($recvstring,length($spi_data)/2,2) ne chr(0x0D).chr(0x0A)){	
		print("Raw SPI RX error...\n");
		print("count: ",$count_in,"\n");
		print("data: \n",unpack("H*",$recvstring),"\n");
		exit(-1);
	}
	
	return(unpack("H*",substr($recvstring, 0, length($spi_data)/2)));
} # end RAW_SPI

###############################################
# this function sets up the serial port
###############################################
sub SETUP_SERIAL{
	my $PortName = $_[0];
	my $COMPORT = "";

	require 5.003;
	if($^O =~ m/MSWIN32/i){
		#  ppm install http://www.bribes.org/perl/ppm/Win32-API.ppd
		require Win32::SerialPort; #(more info at http://members.aol.com/Bbirthisel/SerialPort.html)
		import Win32::SerialPort qw( :STAT 0.19 );

		# open the serial port  (more info at http://members.aol.com/Bbirthisel/SerialPort.html)
		$COMPORT = new Win32::SerialPort ($PortName) || die("Can't open ".$PortName.": ".$^E."\n");  
	}elsif($^O =~ m/linux/i){
		require Device::SerialPort; 
		import Device::SerialPort qw(:PARAM :STAT 0.07);
		# open the serial port 
		$COMPORT = new Device::SerialPort ($PortName) || die("Can't open ".$PortName.": ".$^E."\n");
	}else{
		print("Unknown operating system ",$^O," exiting.\n");
		exit(-1);
	}
	
	# configure the port
	$COMPORT->baudrate(115200);
#	$COMPORT->parity("even");
	$COMPORT->parity("none");
	$COMPORT->databits(8);
	$COMPORT->stopbits(1);
	$COMPORT->handshake("");
	$COMPORT->binary(T);          		# just say Yes (Win 3.x option)
#	$COMPORT->parity_enable(1);   
	if($^O =~ m/MSWIN32/i){
		$COMPORT->debug(0);	# prints msg in linux
		$COMPORT->write_char_time(50);		# needed for write timeouts
		$COMPORT->write_const_time(1000);	# needed for write timeouts
		$COMPORT->read_interval(1000);    	# needed for read timeouts max time between read char (milliseconds)
		$COMPORT->read_char_time(5);     	# needed for read timeouts avg time between read char
	}
	$COMPORT->read_const_time(1000);  	# needed for read timeouts total = (avg * bytes) + const 
	$COMPORT->buffers(4096, 4096);
	$COMPORT->write_settings;		# actualy use the settings above

	return(0,$COMPORT);
}


