# this perl script should take a intel merged hex file and load it onto a pic micro using my IAP update routine (I hope) 
# 26MAY2012 PJS updated script to handel MPLABx (32 bit intel hex files)                          

if($ARGV[0] eq "" || $ARGV[1] eq ""){
	print("Usage:\nperl update_firmware.pl <serial port> <Intel Merged Hex file to load>\n\n");
	exit(-1);
	}

$cardname = "CANSNF";
$serial_port = $ARGV[0];
$hexfile = $ARGV[1];

######################################################################
# init serial stuff
print("Init serial port \n");
@rc = SETUP_SERIAL($serial_port);
if($rc[0] != 0){
	exit($rc[0]);
}
$COMPORT = $rc[1];

#flush serial port buffers
$sendstring = "         ";	# garbage chars at start are to just flush out port
$count_out = $COMPORT->write($sendstring);
if(length($sendstring) != $count_out){
	print("Error send failed!!\n",$count_out," Sent out of ",length($sendstring),"\n");
	exit(-1);
}
# this code clears the read buffer (just read all the chars in until its empty)
$recv_length = 1;
$count_in = 1;
while ($count_in == 1){						
	($count_in, $recvstring) = $COMPORT->read($recv_length);
}

######################################################################
# Stop the card scanning loop
print("	Done\nStop scan\n");
$sendstring = $cardname.chr(0x08); 	# stop the auto scan of the mcp
$count_out = $COMPORT->write($sendstring);
if(length($sendstring) != $count_out){
	print("TX error on stop scan...\n");
	exit(-1);
}
$recv_length = 2;						# how many chars to get
($count_in, $recvstring) = $COMPORT->read($recv_length);
if($count_in != $recv_length || $recvstring ne (chr(0x0D).chr(0x0A))){	
	print("RX cnt error on stop scan...\n");
	print("count: ",$count_in,"\n");
	print("data: \n",unpack("H*",$recvstring),"\n");
	exit(-1);
}

######################################################################
# read firmare level
print("	Done\nRead inital firmware level\n");
$sendstring = $cardname.chr(0x01); 	# send version command
$count_out = $COMPORT->write($sendstring);
if(length($sendstring) != $count_out){
	print("version level TX error ..\n");
	exit(-1);
}
$recv_length = 8;
($count_in, $recvstring) = $COMPORT->read($recv_length);
if($count_in != $recv_length || substr($recvstring,0,1) ne "V" || substr($recvstring,6,2) ne chr(0x0D).chr(0x0A)){	
	print("version level TX error...\n");
	print("count: ",$count_in,"\n");
	print("data: \n",unpack("H*",$recvstring),"\n");
	exit(-1);
}
print("	Current Level = ",substr($recvstring,0,6),"\n");

######################################################################
# load file
print("	Done\nOpen and load data from file ",$hexfile,"\n");
open(FILEIN, $hexfile) || die("cannot open file ",$hexfile,"!!\n\n");
@imh_in = <FILEIN>;
close(FILEIN);
$line = 0;
$endoffile = 0;
@data_array = ();
$address_bytes = 0;
$upper_address_bytes = 0;
foreach $imh_line(@imh_in){
	$line++;
	chomp($imh_line);
	if($^O =~ m/linux/i){
		$imh_line =~ s/\r$//; # dump the window cage return in linux
	}
	#check for end of file record type (01)
	if($imh_line eq ":00000001FF"){
		$endoffile = 1;
	}else{
		#check for starting :
		if(substr($imh_line,0,1) ne ":"){
			print("Error on line ",$line, " missing :\n",$imh_line,"\n");
			exit(-1);
		}
		#check for record type
		if(substr($imh_line,7,2) !~ m/00|01|02|03|04|05/i){
			print("Error on line ",$line, " invalid record type\n",$imh_line,"\n");
			exit(-1);
		}
		#Get the number of data bytes on this line
		$num_data_bytes = hex(substr($imh_line,1,2));
	#print("	number of data bytes ",$num_data_bytes,"\n");
		#calculate the checksum of the line error if not 0
		$temp_check = 0; 
		for($i = 0; $i < ($num_data_bytes + 5); $i++){
			$temp_check =$temp_check + hex(substr($imh_line,($i*2+1),2));
		}	
		if(substr(unpack("h*",pack("l*",$temp_check)),0,2) ne "00"){
			print("Checksum error on line ",$line, " \n",$imh_line,"\n");
			exit(-1);
		}
		
		
		# record type 00 (data record)
		if(substr($imh_line,7,2) =~ m/00/i){
			#Get the address 	 
			$address_bytes = hex(substr($imh_line,3,4)) + $upper_address_bytes; 
		#print("	address bytes ",$address_bytes + $upper_address_bytes,"\n");
			#Get the data	 
			$data_bytes = substr($imh_line,9,($num_data_bytes * 2));
		#print("	data bytes ",$data_bytes,"\n");
	
			for($j = 0; $j < $num_data_bytes; $j++){
				$data_array[$address_bytes + $upper_address_bytes] = substr($data_bytes,($j * 2),2);
				$address_bytes++;
			}	
		# record type 04 (extended linear address record)	
		}elsif(substr($imh_line,7,2) =~ m/04/i){
			if($num_data_bytes != 2){
				print("Error on line ",$line, " invalid lenght for 04 record type\n",$imh_line,"\n");
				exit(-1);
			}
			if(substr($imh_line,3,4) !~m/0000/i){
				print("Error on line ",$line, " invalid address bytes for 04 record type\n",$imh_line,"\n");
				exit(-1);
			}
				
			#Get the address 	 
			$upper_address_bytes = hex(substr($imh_line,9,4)."0000"); 
		#print("	Upper address bytes ",$upper_address_bytes,"\n");
			
			
		}else{
			print("Error on line ",$line, " unsupported record type\n",$imh_line,"\n");
			exit(-1);		
		}
	}
}
if($endoffile != 1){
	print("end of file marker not found!!\n");
	exit(-1);
}


######################################################################
#scan data array and check length if more than 4095 words warn and drop the overage
print("	Done\nPrep data \n");
if(scalar(@data_array) > 8192){
	print("\n	WARNING:\n	more data than what will fit in part found.\n	dropping:\n");
}
$last_valid_byte = 0;
for($i = 0;$i < scalar(@data_array);$i = $i + 2){
	if($i < 8192 && $data_array[$i] ne ""){
		$last_valid_byte = $i + 2;
	}elsif($data_array[$i] ne "" ){
		print("		address = ",unpack("H8",pack("N*",$address))," data = ");
		print($data_array[$i+1],$data_array[$i],"\n");
	}
	$address++;
}
if(scalar(@data_array) > 8192){
	print("\n");
}
# for debug only to force full block write -> $last_valid_byte = 8192;

for($i = 0;$i < $last_valid_byte + 8;$i = $i+2){  # swap bytes and set all empty bytes to FF to save wear on FLASH and to make sure code ends on a boundry
	if($data_array[$i] eq "" || $data_array[$i+1] eq "" ){
		$data_array_pgm[$i] = "3F";
		$data_array_pgm[$i+1] = "FF";
	}else{
		$data_array_pgm[$i] = $data_array[$i+1];
		$data_array_pgm[$i+1] = $data_array[$i];
	}
}

######################################################################
#set start write address to card
print("	Done\nSending data to temporary flash area\n");
$sendstring = $cardname.chr(0x02).chr(0x10).chr(0x00); 	# send temp FLASH start address
$count_out = $COMPORT->write($sendstring);
if(length($sendstring) != $count_out){
	print("temp FLASH address TX error ..\n");
	exit(-1);
}
$recv_length = 2;
($count_in, $recvstring) = $COMPORT->read($recv_length);
if($count_in != $recv_length || $recvstring ne (chr(0x0D).chr(0x0A))){	
	print("temp FLASH address set error...\n");
	print("count: ",$count_in,"\n");
	print("data: \n",unpack("H*",$recvstring),"\n");
	exit(-1);
}

######################################################################
#send data up to card
#14 bits per address location so take the "address" from the file convert to dec and divide by 2
#data is stored low byte high byte so will need to flip before sending to the micro
$address = 0;
for($i = 0;$i < $last_valid_byte;$i = $i + 8){
	$tempdata = chr(hex($data_array_pgm[$i])).chr(hex($data_array_pgm[$i+1])).chr(hex($data_array_pgm[$i+2])).chr(hex($data_array_pgm[$i+3])).chr(hex($data_array_pgm[$i+4])).chr(hex($data_array_pgm[$i+5])).chr(hex($data_array_pgm[$i+6])).chr(hex($data_array_pgm[$i+7]));
	$sendstring = $cardname.chr(0x03).$tempdata; 	# send data to temp FLASH
		$count_out = $COMPORT->write($sendstring);
	if(length($sendstring) != $count_out){
		print("temp FLASH write TX error ..\n");
		exit(-1);
	}
	$recv_length = 3;
	($count_in, $recvstring) = $COMPORT->read($recv_length);
	if($count_in != $recv_length || $recvstring ne ("P".chr(0x0D).chr(0x0A)) ){	
		print("temp FLASH write error ",$address,"...\n");
		print("count: ",$count_in,"\n");
		print("data: \n",unpack("H*",$recvstring),"\n");
		print("\naddress = ",substr(unpack("H8",pack("N*",$address)),4,4)," data = ");
		print($data_array_pgm[$i],$data_array_pgm[$i+1],$data_array_pgm[$i+2],$data_array_pgm[$i+3],$data_array_pgm[$i+4],$data_array_pgm[$i+5],$data_array_pgm[$i+6],$data_array_pgm[$i+7],"\n");	
		exit(-1);
	}

	$address = $address + 4;
}


######################################################################
#read data back from card
print("	Done\nVerifing temporary flash area\n");
$j = 0;
$data_out = "";
while($j <= $last_valid_byte/2 - 1){	# this is needed due to the 4096 buffer limit on the serial port in windows	
	#set address
	$sendstring = $cardname.chr(0x02).chr(hex(substr(unpack("H8",pack("N*",$j + hex(1000))),4,2))).chr(hex(substr(unpack("H8",pack("N*",$j + hex(1000))),6,2))); 	# send temp FLASH start address
	$count_out = $COMPORT->write($sendstring);
	if(length($sendstring) != $count_out){
		print("temp FLASH address TX error ..\n");
		exit(-1);
	}
	$recv_length = 2;
	($count_in, $recvstring) = $COMPORT->read($recv_length);
	if($count_in != $recv_length || $recvstring ne (chr(0x0D).chr(0x0A))){	
		print("temp FLASH address set error...\n");
		print("count: ",$count_in,"\n");
		print("data: \n",unpack("H*",$recvstring),"\n");
		exit(-1);
	}

	if(($j+1024) - $last_valid_byte/2 < 0){
		$temp_length = 1023;
	}else{
		$temp_length = ($last_valid_byte/2 - $j)-1;
	}  
	#set address
	$sendstring = $cardname.chr(0x04).chr(hex(substr(unpack("H8",pack("N*",$temp_length)),4,2))).chr(hex(substr(unpack("H8",pack("N*",$temp_length)),6,2))); 	# read back temp FLASH 
	$count_out = $COMPORT->write($sendstring);
	if(length($sendstring) != $count_out){
		print("Read temp flash TX error ..\n");
		exit(-1);
	}
	$recv_length = ($temp_length+2)*2;	# how many chars to get (tack on the "0x0d0a")

	($count_in, $recvstring) = $COMPORT->read($recv_length);
	if($count_in != $recv_length || substr($recvstring,$recv_length-2,2) ne (chr(0x0D).chr(0x0A))){	
		print("Read temp flash RX error...\n");
		print("count: ",$count_in,"\n");
		#print("data: \n",unpack("H*",$recvstring),"\n");
		exit(-1);
	}
	$data_out = $data_out.substr($recvstring,0,$recv_length-2);
	$j = $j + 1024;
}
for($i = 0; $i < $last_valid_byte; $i++){
	if(uc(unpack("H*",substr($data_out,($i),1))) ne uc($data_array_pgm[$i])){
		print("\nERROR!!\nbyte ",$i," mismatch on verify before burn!\nexp = ",uc($data_array_pgm[$i])," act = ",uc(unpack("H*",substr($data_out,($i),1))),"\n");
		exit(-1);
	}
}


###############################################################
# do the actual burn here
print("	Done\nBurn to active flash area\n");
$sendstring = $cardname.chr(0x05)."BURN"; 	# send flash update command
$count_out = $COMPORT->write($sendstring);
if(length($sendstring) != $count_out){
	print("FLASH update TX error ..\n");
	exit(-1);
}
$recv_length = 8;
($count_in, $recvstring) = $COMPORT->read($recv_length);
if($count_in != $recv_length || $recvstring ne ("BURNING ") ){	
	print("FLASH UPDATE RX error...\n");
	print("count: ",$count_in,"\n");
	print("data: \n",unpack("H*",$recvstring),"\n");
	exit(-1);
}
print("\nNOW burning active portion of flash!\nDO NOT UNPLUG!\nPlease wait\n\n");
sleep(10);
$recv_length = 7;
($count_in, $recvstring) = $COMPORT->read($recv_length);
if($count_in != $recv_length || $recvstring ne ("RESET".chr(0x0D).chr(0x0A)) ){	
	print("RX cnt error could not init box for some reason...\n");
	print("count: ",$count_in,"\n");
	print("data: \n",unpack("H*",$recvstring),"\n");
	exit(-1);
}

######################################################################
#read data back from card
print("	Done\nVerifing flash area\n");
$j = 0;
$data_out = "";
while($j <= $last_valid_byte/2 - 1){	# this is needed due to the 4096 buffer limit on the serial port in windows	
	#set address
	$sendstring = $cardname.chr(0x02).chr(hex(substr(unpack("H8",pack("N*",$j)),4,2))).chr(hex(substr(unpack("H8",pack("N*",$j)),6,2))); 	# send main FLASH start address
	$count_out = $COMPORT->write($sendstring);
	if(length($sendstring) != $count_out){
		print("temp FLASH address TX error ..\n");
		exit(-1);
	}
	$recv_length = 2;
	($count_in, $recvstring) = $COMPORT->read($recv_length);
	if($count_in != $recv_length || $recvstring ne (chr(0x0D).chr(0x0A))){	
		print("temp FLASH address set error...\n");
		print("count: ",$count_in,"\n");
		print("data: \n",unpack("H*",$recvstring),"\n");
		exit(-1);
	}

	if(($j+1024) - $last_valid_byte/2 < 0){
		$temp_length = 1023;
	}else{
		$temp_length = ($last_valid_byte/2 - $j)-1;
	}  
	#set address
	$sendstring = $cardname.chr(0x04).chr(hex(substr(unpack("H8",pack("N*",$temp_length)),4,2))).chr(hex(substr(unpack("H8",pack("N*",$temp_length)),6,2))); 	# read back temp FLASH start addres (0x0003)
	$count_out = $COMPORT->write($sendstring);
	if(length($sendstring) != $count_out){
		print("Read temp flash TX error ..\n");
		exit(-1);
	}
	$recv_length = ($temp_length+2)*2;	# how many chars to get (tack on the "0x0d0a")

	($count_in, $recvstring) = $COMPORT->read($recv_length);
	if($count_in != $recv_length || substr($recvstring,$recv_length-2,2) ne (chr(0x0D).chr(0x0A))){	
		print("Read temp flash RX error...\n");
		print("count: ",$count_in,"\n");
		#print("data: \n",unpack("H*",$recvstring),"\n");
		exit(-1);
	}
	$data_out = $data_out.substr($recvstring,0,$recv_length-2);
	$j = $j + 1024;
}
for($i = 0; $i < $last_valid_byte; $i++){
	if(uc(unpack("H*",substr($data_out,($i),1))) ne uc($data_array_pgm[$i])){
		print("\nERROR!!\nbyte ",$i," mismatch on verify before burn!\nexp = ",uc($data_array_pgm[$i])," act = ",uc(unpack("H*",substr($data_out,($i),1))),"\n");
		exit(-1);
	}
}

######################################################################
# read firmare level
print("	Done\nRead final firmware level\n");
$sendstring = $cardname.chr(0x01); 	# send version command
$count_out = $COMPORT->write($sendstring);
if(length($sendstring) != $count_out){
	print("version level TX error ..\n");
	exit(-1);
}
$recv_length = 8;
($count_in, $recvstring) = $COMPORT->read($recv_length);
if($count_in != $recv_length || substr($recvstring,0,1) ne "V" || substr($recvstring,6,2) ne chr(0x0D).chr(0x0A)){	
	print("version level TX error...\n");
	print("count: ",$count_in,"\n");
	print("data: \n",unpack("H*",$recvstring),"\n");
	exit(-1);
}
print("	Level = ",substr($recvstring,0,6),"\n");

print("	done\nFlash compleate!\n\n");

exit(0);




###############################################
# this function sets up the serial port
###############################################
sub SETUP_SERIAL{
	my $PortName = $_[0];
	my $COMPORT = "";

	require 5.003;
	if($^O =~ m/MSWIN32/i){
		require Win32::SerialPort; #(more info at http://members.aol.com/Bbirthisel/SerialPort.html)
		import Win32::SerialPort qw( :STAT 0.19 );

		# open the serial port  (more info at http://members.aol.com/Bbirthisel/SerialPort.html)
		$COMPORT = new Win32::SerialPort ($PortName) || die("Can't open ".$PortName.": ".$^E."\n");  
	}elsif($^O =~ m/linux/i){
		require Device::SerialPort; 
		import Device::SerialPort qw(:PARAM :STAT 0.07);
		# open the serial port 
		$COMPORT = new Device::SerialPort ($PortName) || die("Can't open ".$PortName.": ".$^E."\n");
	}else{
		print("Unknown operating system ",$^O," exiting.\n");
		exit(-1);
	}
	
	# configure the port
	$COMPORT->baudrate(115200);
#	$COMPORT->parity("even");
	$COMPORT->parity("none");
	$COMPORT->databits(8);
	$COMPORT->stopbits(1);
	$COMPORT->handshake("");
	$COMPORT->binary(T);          		# just say Yes (Win 3.x option)
#	$COMPORT->parity_enable(1);   
	if($^O =~ m/MSWIN32/i){
		$COMPORT->debug(0);	# prints msg in linux
		$COMPORT->write_char_time(50);		# needed for write timeouts
		$COMPORT->write_const_time(1000);	# needed for write timeouts
		$COMPORT->read_interval(1000);    	# needed for read timeouts max time between read char (milliseconds)
		$COMPORT->read_char_time(5);     	# needed for read timeouts avg time between read char
	}
	$COMPORT->read_const_time(1000);  	# needed for read timeouts total = (avg * bytes) + const 
	$COMPORT->buffers(4096, 4096);
	$COMPORT->write_settings;		# actualy use the settings above

	return(0,$COMPORT);
}


